'use strict';

var _chai = require('chai');

var _createState = require('../src/createState');

var _createState2 = _interopRequireDefault(_createState);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('createState', function () {

  it('should create state with shape values and getters', function () {
    var shape = {
      alpha: 1,
      bravo: 2,
      charlie: 3,

      echo: function echo() {
        return this.delta + this.charlie;
      },
      delta: function delta() {
        return this.alpha + this.bravo;
      }
    };

    var state = (0, _createState2.default)(shape);

    (0, _chai.expect)(state.alpha).to.equal(shape.alpha);
    (0, _chai.expect)(state.bravo).to.equal(shape.bravo);
    (0, _chai.expect)(state.charlie).to.equal(shape.charlie);
    (0, _chai.expect)(state.delta).to.equal(shape.alpha + shape.bravo);
    (0, _chai.expect)(state.echo).to.equal(shape.alpha + shape.bravo + shape.charlie);
  });

  it('should copy state and change values partially', function () {

    var shape = {
      alpha: 1,
      bravo: 2,
      charlie: function charlie() {
        return this.alpha + this.bravo;
      }
    };

    var firstState = (0, _createState2.default)(shape);
    var secondState = firstState.copy({ alpha: 2 });

    (0, _chai.expect)(firstState.alpha).to.equal(1);
    (0, _chai.expect)(firstState.bravo).to.equal(2);
    (0, _chai.expect)(firstState.charlie).to.equal(3);
    (0, _chai.expect)(secondState.alpha).to.equal(2);
    (0, _chai.expect)(secondState.bravo).to.equal(2);
    (0, _chai.expect)(secondState.charlie).to.equal(4);
  });

  it('should recalculate getter after dependency change', function () {
    var bravoCalls = 0;

    var shape = {
      alpha: 1,
      bravo: function bravo() {
        bravoCalls++;
        return this.alpha;
      }
    };

    var firstState = (0, _createState2.default)(shape);

    (0, _chai.expect)(firstState.bravo).to.equal(1);
    (0, _chai.expect)(bravoCalls).to.equal(1);

    var secondState = firstState.copy({ alpha: 2 });

    (0, _chai.expect)(secondState.bravo).to.equal(2);
    (0, _chai.expect)(secondState.bravo).to.equal(2);
    (0, _chai.expect)(bravoCalls).to.equal(2);
  });

  it('should not recalculate getter if dependency did not change', function () {
    var charlieCalls = 0;

    var shape = {
      alpha: 1,
      bravo: 2,
      charlie: function charlie(_ref) {
        var alpha = _ref.alpha;

        charlieCalls++;
        return alpha;
      }
    };

    var firstState = (0, _createState2.default)(shape);
    var secondState = firstState.copy({ bravo: 5 });
    var thirdState = secondState.copy({ bravo: 6, alpha: 1 });

    (0, _chai.expect)(firstState.charlie).to.equal(shape.alpha);
    (0, _chai.expect)(secondState.charlie).to.equal(shape.alpha);
    (0, _chai.expect)(thirdState.charlie).to.equal(shape.alpha);
    (0, _chai.expect)(charlieCalls).to.equal(1);
  });

  it('should cache getter call after recalculation', function () {
    var bravoCalls = 0;

    var shape = {
      alpha: 1,
      bravo: function bravo() {
        bravoCalls++;
        return this.alpha;
      }
    };

    var firstState = (0, _createState2.default)(shape);

    (0, _chai.expect)(firstState.bravo).to.equal(shape.alpha);
    (0, _chai.expect)(firstState.bravo).to.equal(shape.alpha);
    (0, _chai.expect)(bravoCalls).to.equal(1);

    var secondState = firstState.copy({ alpha: 2 });

    (0, _chai.expect)(secondState.bravo).to.equal(2);
    (0, _chai.expect)(secondState.bravo).to.equal(2);
    (0, _chai.expect)(bravoCalls).to.equal(2);
  });
});